Activity {
  include moving;
  successor welding;  
  successor idle;     
}

FactorModel {
  // latent causes
  HRW desc "(H)uman arm and (R)obot on shared (W)orkbench"
    /* requires (HW) */
    prevents (HC) // assumes one operator at a time using the work cell
    mitPreventsMit (HS,HC)
    guard "hSM_PERM & hACT_WORKING & rloc=sharedTbl & hloc=sharedTbl"
    detectedBy (SHARE.HRWdet)
    mitigatedBy (.HRWmit2,.HRWmit3) // ,PREVENT.HRWmit
    resumedBy (.HRWres)
    mis=h_placeWorkpieceLeft|h_grabRightWorkpiece
    prob=0.1
    sev=5;
}

Application cobot {
  mode RCdet desc "touch/vibration sensor"
    guard "hSM_PERM & hACT_WORKING";
  
  mode RTdet desc "touch and image sensors"
    guard "hSM_PERM & hACT_WORKING";
  mode RTmis
    target (safmod=stopped);
  mode RTres 
    target (safmod=normal);

  // detectors, mitigations, and resumptions
  mode HRWdet 
    guard "hSM_PERM & hACT_WORKING & rloc=sharedTbl & lgtBar=true"
    embodiedBy cellObSys;
  mode HRWmit desc "safety-rated monitored stop in current activity"
    cause "hST_HOatShTbl"
    update "(notif_leaveWrkb'=true)" // application of safety function
    target (safmod=srmst) // mitigation to safety mode
    embodiedBy robotArm
    disruption=5
    nuisance="alarmIntensity2 * 2"
    effort=4;
  mode HRWmit2 desc "power/force limitation in current activity"
    cause "hST_HOatShTbl"
    update "(notif_leaveWrkb'=true)"
    target (safmod=pflim)
    embodiedBy robotArm
    disruption=2
    nuisance="alarmIntensity2 * 4"
    effort=7;
  mode HRWmit3 desc "safety-rated monitored stop in current activity"
    cause "hST_HOatShTbl"
    update "(notif_leaveWrkb'=true)"
    target (safmod=stopped)
    embodiedBy robotArm
    disruption=5
    nuisance="alarmIntensity2 * 2"
    effort=4;
  mode HRWres
    cause "hST_HOatShTbl" // used in checks for hazard removal
    update "(notif_leaveWrkb'=false)" // switching off safety function
    target (safmod=normal); // resumption to less restrictive safety mode

  mode HWdet 
    guard "hSM_PERM & hACT_WORKING & lgtBar=true" 
    embodiedBy lgtBar;
  mode HWmit desc "speed/separation monitoring" type LIMIT
    target (safmod=ssmon)
    disruption=1
    effort=2;
  mode HWres 
    cause "hloc!=sharedTbl"
    target (safmod=normal); // target unless unacceptable in reached risk state
}

// risk_X = if X is active, guard by default includes the causes of X
Weights rewards {
                         guard             risk_HRW;
  // agent: robotArm
 r_moveToTable:          ""                "3";
 r_grabLeftWorkpiece: 	 ""                "4";
 r_placeWorkpieceRight:  ""                "4";
 r_moveToWelder: 	       ""                "1";
  // agent: weldingMachine
 rw_weldStep: 		       ""                "0"; 
 rw_leaveWelder: 	       ""                "0";
  // agent: human
 h_idle:                 ""                "1";
 h_start: 		           ""                "0";
 h_placeWorkpieceLeft: 	 ""                "4";
 h_withDrawArm: 	       ""                "2";
 h_grabRightWorkpiece:   ""                "6";
 h_enterCell:		         ""                "0";
 h_approachWeldSpot: 	   ""                "0"; 
 h_exitPlant: 		       ""                "0";
}
