Activity {
  include base;
}

FactorModel {
  // latent causes of accidents
  HS desc "(H)uman in (S)afeguarded area while robot working or welding"
    guard "hSM_PERM & hACT_WORKING & (hloc=inCell | hloc=atWeldSpot)"
    detectedBy (SHARE.HSdet)
    mitigatedBy (.ssmon,.pflim,.stopped) // ,.srmst
    resumedBy (.HSres)
    mis="h_exitPlant|h_approachWeldSpot"
    prob=0.1
    sev=7;
  HC desc "(H)uman (C)lose to active welder and robot working"
    requiresOcc (HS) // imposes relationship between cause/guard conditions of HS and HC, e.g. not HC => not HS
    mitPreventsMit (HS)
    /* prevents (HRW) */ // comment out if two operators are in workcell
    guard "hSM_PERM & hACT_WELDING & hloc=atWeldSpot" // plant + env state without sensor
    detectedBy (SHARE.HCdet)
    mitigatedBy (.HCHguidAud,SHUTDOWN.HCStOffVis,SHUTDOWN.HCSrmstIdleVis) // ,SHUTDOWN.HCStOffAud
    resumedBy (NOTIFY.HCres,NOTIFY.HCres2)
    mis="h_exitPlant"
    prob=0.2 // prob of mishap if unrecognised OR active and not mitigated
    sev=9;
}

Application cobot {
  hACT_WORKING = "(ract=exchWrkp | ract=welding | wact=welding)";
  hACT_IDLE = "!hACT_WORKING";

  // detectors, mitigation and resumption commands ...
  mode HSdet desc "range detector"
    guard "hSM_PERM & hACT_WORKING & (rngDet=near | rngDet=close)";
  mode ssmon desc "speed/separation monitoring / audio warning"
    event ssmon
    cause "hST_HOinSGA" // optional
    update "(notif'=leaveArea)" // optional
    target (safmod=ssmon) // no activity change
    disruption=3 // low disruption
    nuisance="alarmIntensity1 * 6" // moderate nuisance
    effort=7.5; // wear/tear intensive
  mode srmst desc "safety/rated monitored stop / visual warning"
    event srmst
    cause "hST_HOinSGA" 
    update "(notif'=leaveArea)" 
    target (safmod=srmst) // act=idle
    disruption=7.5 // highly disruptive 
    nuisance="alarmIntensity1 * 4" // moderate nuisance
    effort=4.5; // moderate wear/tear
  mode pflim desc "power and force limiting mode / loud audio warning"
    event pflim
    cause "hST_HOinSGA" 
    update "(notif'=leaveArea)" 
    target (safmod=pflim)
    disruption=2.5 // little disruptive 
    nuisance="alarmIntensity1 * 8" // high nuisance
    effort=4.5; // moderate wear/tear
  mode stopped desc "protective emergency stop / minor alarm"
    event stopped
    cause "hST_HOinSGA" 
    update "(notif'=leaveArea)" 
    target (safmod=stopped)
    disruption=9 // ultimately disruptive
    nuisance="alarmIntensity1 * 1" // operator should be able to perform maintenance
    effort=5.5; // moderate wear/tear
  mode HSres
    cause "hST_HOinSGA"
    target (safmod=normal) // act=exchWrkp
    disruption=0
    nuisance=0
    effort=2;

  mode HCdet desc "light barrier"
    guard "hSM_PERM & hACT_WELDING & rngDet=close" 
    embodiedBy rngDet;
  mode HCStOffVis
    desc "emergency stop of cell / robot+welder turned off / visual notif."
    // a. synchronise with all actors on transition to act
    event stop 
    // c. apply safety function
    cause "hST_HOinSGA" update "(notif'=leaveArea)"
    // d. set activity and safety mode as the mitigation target
    target (act=off, safmod=stopped)
    // e. specify mitigation characteristics
    disruption=9
    nuisance="alarmIntensity1 * 5" // wear/tear intensive, maintenance effort, energy
    effort=5.5; 
  mode HCSrmstIdleVis desc "safety-rated monitored stop of robot / robot+welder go idle / visual notif."
    event halt cause "hST_HOinSGA"
    update "(notif'=leaveArea)" target (act=idle, safmod=srmst)
    disruption=7.5
    nuisance="alarmIntensity1 * 6"
    effort=6.5;
  mode HCStOffAud desc "emergency stop of cell / robot+welder turned off / audio-vis. notif."
    event stop2 cause "hST_HOinSGA"
    update "(notif'=leaveArea)" target (act=off, safmod=stopped)
    disruption=10
    nuisance="alarmIntensity1 * 9"
    effort=5;
  mode HCHguidAud desc "hand-guided welding / moderate vis. notif."
    cause "hST_HOinSGA"
    update "(notif'=leaveArea)" target (safmod=hguid)
    disruption=2.5
    nuisance="alarmIntensity1 * 1" // let operator perform welding task, little nuisance
    effort=4;
  mode HCres desc "exchange workpiece and start over"
    // a. synchronise with all actors on transition to exchWrkp
    event resume
    // b. causal factor
    cause "hST_HOinSGA"
    // c. deactivate safety function
    update "(notif'=ok)"
    // d. set activity and safety mode as the resumption target
    target (act=exchWrkp, safmod=normal)
    // e. specify resumption characteristics
    disruption=3 // moderate disruption of work process
    nuisance=0
    effort=7.5; // high wear/tear penality
  mode HCres2 desc "continue welding if workpiece still unfinished"
    event resume2
    cause "hST_HOinSGA" update "(notif'=ok)"
    target (act=welding, safmod=normal)
    disruption=6 // negative impact on work piece quality because of disrupted welding procedure
    nuisance=0
    effort=4; // slightly less wear/tear/energy consumption of robot
}


// exposure to mishap from factor X when performing action A while X is active
// risk_X = if X is active, guard by default includes the conditions for X
Weights rewards {
                         guard             risk_HC risk_HS;
  // agent: robotArm
 r_moveToTable:          ""                "5"     "9";
 r_grabLeftWorkpiece: 	 ""                "0"     "3";
 r_placeWorkpieceRight:  ""                "0"     "3";
 r_moveToWelder: 	       ""                "9"     "5";
  // agent: weldingMachine
 rw_weldStep: 		       ""                "10"    "3"; 
 rw_leaveWelder: 	       ""                "5"     "6";
  // agent: human
 h_idle:                 ""                "1"     "1"; 
 h_start: 		           ""                "0"     "0";
 h_placeWorkpieceLeft: 	 ""                "0"     "0";
 h_withDrawArm: 	       ""                "0"     "0";
 h_grabRightWorkpiece:   ""                "0"     "0";
 h_enterCell:		         ""                "0"     "7";
 h_approachWeldSpot: 	   ""                "7"     "4"; 
 h_exitPlant: 		       ""                "2"     "2";
}
