# Supplemental Material on Supervisor Synthesis with YAP

CC-BY-NC 4.0, (c) Mario Gleirscher

This package provides additional materials to run the YAP-driven
synthesis experiments being part of my research about risk structures,
the
[CSI:Cobot](https://www.sheffield.ac.uk/sheffieldrobotics/about/csi-cobots/planning)
case study, and follow-up works.


## Package Content

  1. This README
  2. YAP executable (to be installed from *.deb or unzipped)
  3. YAP manual; see the [installed](/usr/share/yapp/doc/manual.pdf)
     or [unzipped](../../doc/manual.pdf) version
  4. Example files for the case study


## System Requirements

  1. Ubuntu 18.04+
  2. Java (e.g. OpenJDK) installed; 
     on Linux, the example script assumes `/usr/lib/jvm/java-14-openjdk-amd64/bin/java`
  3. PRISM 4.5+ installed; on Linux with the binary
     `/opt/prism/bin/prism`; see the [PRISM
     website](www.prismmodelchecker.org) or on
     [GitHub](https://github.com/prismmodelchecker/prism) for details
  4. EvoChecker installed; 
     on Linux installed in `/opt/EvoChecker`;
	 see the [EvoChecker
     website](https://www-users.cs.york.ac.uk/~simos/EvoChecker/) or
     on [GitHub](https://github.com/gerasimou/EvoChecker) for
	 details (requires JMetal 4.5, PRISM 4.5+, and ANTLR 4)
  5. YAP installed (deb) or unpacked (zip);
	 see the YAP manual [here](/usr/share/yapp/doc/manual.pdf) or
	 [here](../../doc/manual.pdf) for details
  6. (Optional) For test generation:
     [libfsmtest](https://bitbucket.org/JanPeleska/libfsmtest/src/master/)
     and GNU C++ compiler including libstdc++ installed (the
     evaluation package assumes libfsmtest to reside in
     `/opt/libfsmtest` and corresponding shared object in
     `/usr/lib/libfsmtest.so`)
  7. (Optional) Postprocessing tools installed; GraphViz,
     [Matplotlib](https://matplotlib.org/) (needs Python 3)

The experiment has only been run on a Ubuntu 20.04 machine.  Other
Linux variants and Mac might work, Windows won't because of **run.sh**
being a Bash script that needs to be ported.


## Run the Experiment

 1. **Under Linux**: To copy the example files into a writable folder used by 
    the script `run.sh`, use
    
    ```
    mkdir -p ~/yap-examples/hrc2
    cp -r /usr/share/yapp/examples/hrc2 ~/yap-examples/hrc2
    ```
    Eventhough you can run the experiment several times, I recommend to
    make a fresh copy after each run!
 
 2. To run the default steps of the synthesis experiment, call 

	`./run.sh`

	from a Bourne shell.  This Bash script executes YAP, EvoChecker,
	PRISM, and performs further postprocessing steps.  The output is
	stored in sub-folders named "xy-processingstage" where
	*processingstage* should largely explain itself.
    
 3. To generate controller code (C#, C++), a conformance test suite
    for the C++ target, and run these tests against this target, call
 
    `./run.sh -c -r`
    
    Watch the console.  The output is stored in *06-tests*, the test
    log (verdicts) should then be in file *workcell_results-h-0.log*.
    
 4. To run the variant of the experiment (step 2.) where EvoChecker is
    used to *search the whole space of non-deterministic controller
    decisions*, call
    
    `./run.sh -y -i -e -v`
	
**Note 1:** Use *./run.sh -h* to get help on further steps/processing
options.

**Note 2:** If you installed the above components in other places or
under a non-Linux system, you will have to adjust the corresponding
parameters in the "tool/installation settings" section of the file
`run.sh`.

Please, have a look into our
[ICECCS'20](https://ieeexplore.ieee.org/document/9376211),
[FMAS'20](https://arxiv.org/abs/2012.01649), and
[FMAS'21](https://fmasworkshop.github.io/FMAS2021/) papers for further
discussion and examples.
